==> xs-plugins-0.0.4.tgz: package/src/lib/router.js <==
const path = require('path');
const fs = require('fs');
const chokidar = require('chokidar');
const { parse } = require('@vue/component-compiler-utils');
const vueCompiler = require('vue-template-compiler');
const diff = require('deep-diff').diff;
const apiAutoPlugin = require("./api");
const menuPlugin = require('./menu')
class VueAutoRouteWebapckPlugin {
  constructor(options) {
    this.options = Object.assign({
      rootComponent: 'index',
      indexComponent: 'index',
      ignoreDir: 'components',
      propsKeyName: '$$route',
      autoPath: 'src/automatically'
    }, options);
    this.options.entry = path.join(options.rootPath, '/src/modules/' + options.moduleConfig.CUR_MODULES + '/')
    this.files = {};
    this.filesPath = [];
    this.metaOutput = {
      warnings: [],
      errors: []
    };
    this.routes = [];
    this.routesMap = {};
  }

  async apply(compiler) {
    this._hook(compiler, 'emit', 'emit', (compilation, cb) => {
      compilation.errors = compilation.errors.concat(this.metaOutput.errors.map(x => '[webpack-auto-vue-router] ' + x));
      compilation.warnings = compilation.warnings.concat(this.metaOutput.warnings.map(x => '[webpack-auto-vue-router] ' + x));
      cb && cb();
    });
    this._hook(compiler, 'done', 'done', (compilation, cb) => {
      if (!this.watcher && process.env.NODE_ENV === "development") {
        this.watcher = chokidar.watch(this.options.entry, {
          ignoreInitial: true
        });
        let changeCb = this._fileChanged.bind(this);
        let removeCb = this._fileRmoved.bind(this);
        this.watcher.on('add', changeCb).on('change', changeCb).on('unlink', removeCb);
      }
      cb && cb();
    })
    this.start();
  }
  _isVue(path) {
    return /\.vue$/g.test(path);
  }

  _fileChanged(file) {
    let path = file.replace(this.options.entry, '');
    if (path.indexOf(this.options.ignoreDir + '/') != -1) {
      return;
    }
    this._isVue(file) && this._diffConfig(file);
  }

  _fileRmoved(file) {
    (this._isVue(file) || this._isDirectory(file)) && this.start();
  }

  _hook(compiler, v3Name, v4Name, cb) {
    if (compiler.hooks && compiler.hooks[v4Name]) {
      compiler.hooks[v4Name].tap('VueAutoRouteWebapckPlugin', cb);
    } else {
      compiler.plugin(v3Name, cb);
    }
  }

  async start() {
    this.getRouter();
    await this.getApi();
    if(process.env.NODE_ENV === 'production') {  
      this.getMenu(); // 打包时生成的文件
    } else {
        let  filePath = path.join(this.options.rootPath , '/' + this.options.autoPath + '/menu.js');
        if(!fs.existsSync(filePath)){
            fs.writeFileSync(filePath, '')
        }
    }
  }

  getRouter () {
    let filesPath = this.fetchFiles(this.options.entry);
    this.routes = [];
    this.files = {};
    this.routesMap = {};
    this.filesPath = filesPath;
    this.parseFiles();
    this.buildRoute();
  }
  async getApi () {
    let api = new apiAutoPlugin({...this.options})
    await api.apply();
    this.apiDic = api.getApi()
  }
  getMenu () {
    let menu = new menuPlugin ({...Object.assign(this.options, {router: this.routes, api: this.apiDic})})
    menu.apply();
  }

  _diffConfig(file) {
    let config = this.parseRouteConfig(file);
    let _config = this.files[file];
    if (_config) {
      let hasDiff = diff(_config, config);
      if (hasDiff) {
        this.files[file] = config;
        this.updateOneRoute(config);
      };
    } else {
      this.files[file] = config;
      this.pushOneRoute(config);
      this.output();
    }
  }

  updateOneRoute(config) {
    let {
      _path,
      name,
      meta,
      lazy,
      redirect,
      alias,
      props,
      path: _path_
    } = config;
    let route = this.routesMap[_path];

    if (name !== undefined) route.name = name;
    if (meta !== undefined) route.meta = meta;
    if (redirect !== undefined) route.redirect = redirect;
    if (alias !== undefined) route.alias = alias;
    if (props !== undefined) route.props = props;
    if (_path_ !== undefined) route.path = _path_;
    route.components.default = lazy ? `${lazy}|lazy|${_path}` : _path;
    this.output();
  }

  parseFiles() {
    this.filesPath.forEach(path => {
      this.files[path] = this.parseRouteConfig(path);
    });
  }

  checkDuplicate() {
    let data = {};
    this.metaOutput.warnings = [];
    for (let _path in this.routesMap) {
      let name = this.routesMap[_path].name;
      if (name) {
        if (data[name]) {
          this.metaOutput.warnings.push(`Duplicate named routes definition: [${_path},${data[name]}]`);
        } else {
          data[name] = _path;
        }
      }
    }
    data = null;
  }

  pushOneRoute(config) {
    let pathItems = config._path.replace(this.options.entry, '').split(path.sep);

    this.touchRoute(this.routes, null, pathItems, config);
  }

  buildRoute() {
    this.filesPath.forEach(path => {
      let routeConfig = this.files[path];
      this.pushOneRoute(routeConfig);
    });
    this.output();
  }

  output() {
    let routes = JSON.stringify(this.routes);
    routes = routes.replace(/"default":(".+?\.vue")/g, (v1, v2) => {
      let _v2 = v2.slice(1, -1).replace(/\\+/g, '/').split('|lazy|');
      if (_v2.length === 1) {
        return `default:require('${_v2[0]}').default`;
      } else if (_v2.length === 2) {
        let webpackChunkName = _v2[0] === 'true' ? '' : `/*webpackChunkName:"${_v2[0]}"*/`;
        return `default:()=>import(${webpackChunkName}"${_v2[1]}")`;
      } else {
        return `default:require('${v2}').default`;
      }
    });

    // 2021年04月15日 for songWenGuang
    // 新增 router 文件自动生成 
    var outputPath = path.join(this.options.rootPath , '/' + this.options.autoPath + '/router.js')
    if (!fs.existsSync(outputPath)) {
      var pathtmp;
      var outputPathArry = outputPath.split(/\\/)
      var len = outputPathArry.length;
      outputPathArry.forEach(function (dirname, index) {
        if (index + 1 === len) return;
        pathtmp = pathtmp ? path.join(pathtmp, dirname) : dirname ? dirname : '/'
        if (!fs.existsSync(pathtmp) && !fs.mkdirSync(pathtmp)) {
          return false;
        }
      });
    }
    fs.writeFileSync(outputPath, `export default ${routes}`);
    this.checkDuplicate();
  }


  touchRoute(route, parentRoute, pathItems, routeConfig) {
    let {
      rootComponent,
      indexComponent,
    } = this.options;
    let {
      _path,
      path: __path__,
      meta,
      name,
      lazy,
      redirect,
      alias,
      props
    } = routeConfig;

    // lazy = true;

    let _reoutes = parentRoute && parentRoute.children || route;
    let pathItem = pathItems.shift();
    let components = {};
    if (this._isVue(pathItem)) {
      let _path_ = lazy ? `${lazy}|lazy|${_path}` : _path;
      pathItem = pathItem.replace('.vue', '');
      let len = '/src/modules/'.length
      let winPath = _path_.replace(/\\/g, "/");
      let startIndex = winPath.indexOf('/src/modules/') + len
      let endIndex = winPath.lastIndexOf('/index.vue');
      let mateMenuId = winPath.substring(startIndex, endIndex);

      if (!meta) {
        meta = {}
      }
      meta['MenuId'] = mateMenuId;
      meta['sort'] = parseInt(parentRoute.path.replace('/', ''));

      components.default = _path_;

      if (pathItem === indexComponent) {
        if (parentRoute) {
          parentRoute.components = Object.assign(parentRoute.components, components);
          parentRoute.meta = meta;
          if (redirect !== undefined) parentRoute.redirect = redirect;
          if (lazy !== undefined) parentRoute.lazy = lazy;
          if (alias !== undefined) parentRoute.alias = alias;
          if (props !== undefined) parentRoute.props = props;
          this.routesMap[_path] = parentRoute;
        }
      } else {

        let _route_ = {
          path: __path__ || `${parentRoute ? pathItem : pathItem === rootComponent ? '/' : '/' + pathItem}`,
          components,
          redirect,
          alias,
          props,
          meta
        };
        this.routesMap[_path] = _route_;
        _reoutes.push(_route_);
      }
    } else {
      let _route = _reoutes.find(r => r.path === pathItem || r.path === '/' + pathItem);
      if (!_route) {
        _route = {
          path: __path__ || `${parentRoute ? pathItem : '/' + pathItem}`,
          redirect,
          alias,
          props,
          children: [],
          components: {}
        }
        _reoutes.push(_route);
      }
      this.touchRoute(_route.children, _route, pathItems, routeConfig);
    }
  }

  _isDirectory(path) {
    let stat = fs.statSync(path);
    return stat && stat.isDirectory();
  }

  fetchFiles(dir) {
    let files = [];
    let list = fs.readdirSync(dir);
    list.forEach(file => {
      if (file === this.options.ignoreDir) return;
      let filePath = path.join(dir, file);
      if (this._isDirectory(filePath)) {
        files = files.concat(this.fetchFiles(filePath));
      } else {
        if (this._isVue(file)) {
          files.push(filePath);
        }
      }
    });
    return files;
  }

  parseRouteConfig(filePath) {
    const descriptor = parse({
      source: fs.readFileSync(filePath, {
        encoding: 'utf8'
      }),
      compiler: vueCompiler,
      filename: path.basename(filePath)
    });
    let route;
    let _config = {
      meta: null,
      lazy: false,
      view: null,
      _path: filePath
    };
    let script = descriptor.script ? descriptor.script.content : null;
    if (script) {
      script = script.replace(/\/\/[\S\s]+?[\r\n]/g, '').replace(/\/\*[\s\S]+?\*\//g, '');
      route = this.parseRoute(script);
      if (route) {
        try {
          route = new Function('return ' + route.replace(/[\r\n]+/g, ''))();
          route = Object.assign(_config, route);
        } catch (error) {
          console.error(route);
          throw new Error(error);
        }
      } else {
        route = _config;
      }
    } else {
      route = _config;
    }
    return route;
  }

  parseRoute(txt) {
    let data = txt;
    let mark = this.options.propsKeyName + ':';
    let chart1 = '{';
    let chart2 = '}';
    let startIndex = data.indexOf(mark);
    let count = 0;
    let begingIndex;
    let endingIndex;
    if (startIndex === -1) {
      return '';
    }
    for (let i = startIndex; i < data.length; i++) {
      let w = data.charAt(i);
      if (w === chart1) {
        if (count === 0) begingIndex = i;
        ++count;
      } else if (w === chart2) {
        --count;
        if (count === 0) {
          endingIndex = i + 1;
          break;
        }
      }
    }
    return endingIndex ? data.slice(begingIndex, endingIndex) : '';
  }
}

module.exports = VueAutoRouteWebapckPlugin;

==> xs-plugins-0.0.12.tgz: package/src/lib/router.js <==
const path = require('path');
const fs = require('fs');
const chokidar = require('chokidar');
const { parse } = require('@vue/component-compiler-utils');
const vueCompiler = require('vue-template-compiler');
const diff = require('deep-diff').diff;
const apiAutoPlugin = require("./api");
const menuPlugin = require('./menu')
class VueAutoRouteWebapckPlugin {
  constructor(options) {
    this.options = Object.assign({
      rootComponent: 'index',
      indexComponent: 'index',
      ignoreDir: 'components',
      propsKeyName: '$$route',
      autoPath: 'src/automatically'
    }, options);
    this.options.entry = path.join(options.rootPath, '/src/modules/' + options.moduleConfig.CUR_MODULES + '/')
    this.files = {};
    this.filesPath = [];
    this.metaOutput = {
      warnings: [],
      errors: []
    };
    this.routes = [];
    this.routesMap = {};
  }

  async apply(compiler) {
    this._hook(compiler, 'emit', 'emit', (compilation, cb) => {
      compilation.errors = compilation.errors.concat(this.metaOutput.errors.map(x => '[webpack-auto-vue-router] ' + x));
      compilation.warnings = compilation.warnings.concat(this.metaOutput.warnings.map(x => '[webpack-auto-vue-router] ' + x));
      cb && cb();
    });
    this._hook(compiler, 'done', 'done', (compilation, cb) => {
      if (!this.watcher && process.env.NODE_ENV === "development") {
        this.watcher = chokidar.watch(this.options.entry, {
          ignoreInitial: true
        });
        let changeCb = this._fileChanged.bind(this);
        let removeCb = this._fileRmoved.bind(this);
        this.watcher.on('add', changeCb).on('change', changeCb).on('unlink', removeCb);
      }
      cb && cb();
    })
    this.start();
  }
  _isVue(path) {
    return /\.vue$/g.test(path);
  }

  _fileChanged(file) {
    let path = file.replace(this.options.entry, '');
    if (path.indexOf(this.options.ignoreDir + '/') != -1) {
      return;
    }
    this._isVue(file) && this._diffConfig(file);
  }

  _fileRmoved(file) {
    (this._isVue(file) || this._isDirectory(file)) && this.start();
  }

  _hook(compiler, v3Name, v4Name, cb) {
    if (compiler.hooks && compiler.hooks[v4Name]) {
      compiler.hooks[v4Name].tap('VueAutoRouteWebapckPlugin', cb);
    } else {
      compiler.plugin(v3Name, cb);
    }
  }

  async start() {
    this.getRouter();
    await this.getApi();
    if(process.env.NODE_ENV === 'production') {  
      this.getMenu(); // 打包时生成的文件
    } else {
        let  filePath = path.join(this.options.rootPath , '/' + this.options.autoPath + '/menu.js');
        if(!fs.existsSync(filePath)){
            fs.writeFileSync(filePath, '')
        }
    }
  }

  getRouter () {
    let filesPath = this.fetchFiles(this.options.entry);
    this.routes = [];
    this.files = {};
    this.routesMap = {};
    this.filesPath = filesPath;
    this.parseFiles();
    this.buildRoute();
  }
  async getApi () {
    let api = new apiAutoPlugin({...this.options})
    await api.apply();
    this.apiDic = api.getApi()
  }
  getMenu () {
    let menu = new menuPlugin ({...Object.assign(this.options, {router: this.routes, api: this.apiDic})})
    menu.apply();
  }

  _diffConfig(file) {
    let config = this.parseRouteConfig(file);
    let _config = this.files[file];
    if (_config) {
      let hasDiff = diff(_config, config);
      if (hasDiff) {
        this.files[file] = config;
        this.updateOneRoute(config);
      };
    } else {
      this.files[file] = config;
      this.pushOneRoute(config);
      this.output();
    }
  }

  updateOneRoute(config) {
    let {
      _path,
      name,
      meta,
      lazy,
      redirect,
      alias,
      props,
      path: _path_
    } = config;
    let route = this.routesMap[_path];

    if (name !== undefined) route.name = name;
    if (meta !== undefined) route.meta = meta;
    if (redirect !== undefined) route.redirect = redirect;
    if (alias !== undefined) route.alias = alias;
    if (props !== undefined) route.props = props;
    if (_path_ !== undefined) route.path = _path_;
    route.components.default = lazy ? `${lazy}|lazy|${_path}` : _path;
    this.output();
  }

  parseFiles() {
    this.filesPath.forEach(path => {
      this.files[path] = this.parseRouteConfig(path);
    });
  }

  checkDuplicate() {
    let data = {};
    this.metaOutput.warnings = [];
    for (let _path in this.routesMap) {
      let name = this.routesMap[_path].name;
      if (name) {
        if (data[name]) {
          this.metaOutput.warnings.push(`Duplicate named routes definition: [${_path},${data[name]}]`);
        } else {
          data[name] = _path;
        }
      }
    }
    data = null;
  }

  pushOneRoute(config) {
    let pathItems = config._path.replace(this.options.entry, '').split(path.sep);

    this.touchRoute(this.routes, null, pathItems, config);
  }

  buildRoute() {
    this.filesPath.forEach(path => {
      let routeConfig = this.files[path];
      this.pushOneRoute(routeConfig);
    });
    this.output();
  }

  output() {
    let routes = JSON.stringify(this.routes);
    routes = routes.replace(/"default":(".+?\.vue")/g, (v1, v2) => {
      let _v2 = v2.slice(1, -1).replace(/\\+/g, '/').split('|lazy|');
      if (_v2.length === 1) {
        return `default:require('${_v2[0]}').default`;
      } else if (_v2.length === 2) {
        let webpackChunkName = _v2[0] === 'true' ? '' : `/*webpackChunkName:"${_v2[0]}"*/`;
        return `default:()=>import(${webpackChunkName}"${_v2[1]}")`;
      } else {
        return `default:require('${v2}').default`;
      }
    });

    // 2021年04月15日 for songWenGuang
    // 新增 router 文件自动生成 
    var outputPath = this.formatPath(path.join(this.options.rootPath , '/' + this.options.autoPath + '/router.js'))
    if (!fs.existsSync(outputPath)) {
      var pathtmp;
      var outputPathArry = outputPath.split(/\//)
      var len = outputPathArry.length;
      outputPathArry.forEach(function (dirname, index) {
        if (index + 1 === len) return;
        pathtmp = pathtmp ? path.join(pathtmp, dirname) : dirname ? dirname : '/'
        if (!fs.existsSync(pathtmp) && !fs.mkdirSync(pathtmp)) {
          return false;
        }
      });
    }
    fs.writeFileSync(outputPath, `export default ${routes}`);
    this.checkDuplicate();
  }


  touchRoute(route, parentRoute, pathItems, routeConfig) {
    let {
      rootComponent,
      indexComponent,
    } = this.options;
    let {
      _path,
      path: __path__,
      meta,
      name,
      lazy,
      redirect,
      alias,
      props
    } = routeConfig;

    // lazy = true;

    let _reoutes = parentRoute && parentRoute.children || route;
    let pathItem = pathItems.shift();
    let components = {};
    if (this._isVue(pathItem)) {
      let _path_ = lazy ? `${lazy}|lazy|${_path}` : _path;
      pathItem = pathItem.replace('.vue', '');
      let len = '/src/modules/'.length
      let winPath = _path_.replace(/\\/g, "/");
      let startIndex = winPath.indexOf('/src/modules/') + len
      let endIndex = winPath.lastIndexOf('/index.vue');
      let mateMenuId = winPath.substring(startIndex, endIndex);

      if (!meta) {
        meta = {}
      }
      meta['MenuId'] = mateMenuId;
      meta['sort'] = parseInt(parentRoute.path.replace('/', ''));

      components.default = _path_;

      if (pathItem === indexComponent) {
        if (parentRoute) {
          parentRoute.components = Object.assign(parentRoute.components, components);
          parentRoute.meta = meta;
          if (redirect !== undefined) parentRoute.redirect = redirect;
          if (lazy !== undefined) parentRoute.lazy = lazy;
          if (alias !== undefined) parentRoute.alias = alias;
          if (props !== undefined) parentRoute.props = props;
          this.routesMap[_path] = parentRoute;
        }
      } else {

        let _route_ = {
          path: __path__ || `${parentRoute ? pathItem : pathItem === rootComponent ? '/' : '/' + pathItem}`,
          components,
          redirect,
          alias,
          props,
          meta
        };
        this.routesMap[_path] = _route_;
        _reoutes.push(_route_);
      }
    } else {
      let _route = _reoutes.find(r => r.path === pathItem || r.path === '/' + pathItem);
      if (!_route) {
        _route = {
          path: __path__ || `${parentRoute ? pathItem : '/' + pathItem}`,
          redirect,
          alias,
          props,
          children: [],
          components: {}
        }
        _reoutes.push(_route);
      }
      this.touchRoute(_route.children, _route, pathItems, routeConfig);
    }
  }

  _isDirectory(path) {
    let stat = fs.statSync(path);
    return stat && stat.isDirectory();
  }

  fetchFiles(dir) {
    let files = [];
    let list = fs.readdirSync(dir);
    list.forEach(file => {
      if (file === this.options.ignoreDir) return;
      let filePath = path.join(dir, file);
      if (this._isDirectory(filePath)) {
        files = files.concat(this.fetchFiles(filePath));
      } else {
        if (this._isVue(file)) {
          files.push(filePath);
        }
      }
    });
    return files;
  }

  parseRouteConfig(filePath) {
    const descriptor = parse({
      source: fs.readFileSync(filePath, {
        encoding: 'utf8'
      }),
      compiler: vueCompiler,
      filename: path.basename(filePath)
    });
    let route;
    let _config = {
      meta: null,
      lazy: false,
      view: null,
      _path: filePath
    };
    let script = descriptor.script ? descriptor.script.content : null;
    if (script) {
      script = script.replace(/\/\/[\S\s]+?[\r\n]/g, '').replace(/\/\*[\s\S]+?\*\//g, '');
      route = this.parseRoute(script);
      if (route) {
        try {
          route = new Function('return ' + route.replace(/[\r\n]+/g, ''))();
          route = Object.assign(_config, route);
        } catch (error) {
          console.error(route);
          throw new Error(error);
        }
      } else {
        route = _config;
      }
    } else {
      route = _config;
    }
    return route;
  }

  parseRoute(txt) {
    let data = txt;
    let mark = this.options.propsKeyName + ':';
    let chart1 = '{';
    let chart2 = '}';
    let startIndex = data.indexOf(mark);
    let count = 0;
    let begingIndex;
    let endingIndex;
    if (startIndex === -1) {
      return '';
    }
    for (let i = startIndex; i < data.length; i++) {
      let w = data.charAt(i);
      if (w === chart1) {
        if (count === 0) begingIndex = i;
        ++count;
      } else if (w === chart2) {
        --count;
        if (count === 0) {
          endingIndex = i + 1;
          break;
        }
      }
    }
    return endingIndex ? data.slice(begingIndex, endingIndex) : '';
  }
  /**
    * 路径统一
    */
   formatPath(p) {
    if (p && typeof p === 'string') {
      const sep = path.sep;
      if (sep === '/')
        return p
      else {
        return p.replace(/\\/g, '/')
      }
    }
    return p
  }
}

module.exports = VueAutoRouteWebapckPlugin;
